@@ -2,11 +2,13 @@
 #' 
 #' @title Bootstraping for generalized nonlinear models and nonlinear mixed models
 #' @name boot_nlme
+#' @rdname boot_nlme
 #' @param object object of class \code{\link[nlme]{nlme}} or \code{\link[nlme]{gnls}}
 #' @param f function to be applied (and bootstrapped), default coef (gnls) or fixef (nlme)
 #' @param R number of bootstrap samples, default 999
 #' @param psim simulation level for vector of fixed parameters either for \code{\link{simulate_gnls}} or \code{\link{simulate_nlme_one}}
 #' @param cores number of cores to use for parallel computation
+#' @param data optional data argument (useful/needed when data are not in an available environment).
 #' @param ... additional arguments to be passed to function \code{\link[boot]{boot}}
 #' @details This function is inspired by \code{\link[car]{Boot}}, which does not
 #' seem to work with 'gnls' or 'nlme' objects. This function makes multiple copies 
@@ -38,6 +40,7 @@ boot_nlme <- function(object,
                       R = 999, 
                       psim = 1, 
                       cores = 1L,
+                      data = NULL,
                       ...){
   ## I chose to write it in this way instead of UseMethod
   ## because I feel it is more efficient and results in less code
@@ -47,8 +50,15 @@ boot_nlme <- function(object,
 
   ## extract the original data
   ## This is needed for 'boot'
-  ## dat <- eval(object$call$data) -- old version
-  dat <- nlme::getData(object)
+  dat <- try(nlme::getData(object), silent = TRUE)
+  if(inherits(dat, "try-error") && missing(data)){
+    stop("'data' argument is required. It is likely you are using boot_nlme inside another function. 
+          The data are not in an available environment.", call. = FALSE)    
+  } 
+  
+  if(!missing(data)){
+    dat <- data
+  }
 
   if(missing(f)){
     if(inherits(object, "gnls")) f <- coef
@@ -148,6 +158,11 @@ boot_nlme <- function(object,
   return(ans)
 }
 
+#' @rdname boot_nlme
+#' @description bootstrap function for objects of class \code{\link[nlme]{gnls}}
+#' @export
+boot_gnls <- boot_nlme
+
 #' Create an nlraa environment for bootstrapping
 #' 
 #' @title Environment to store options and data for nlraa
